"""
indra

The indra module of Indra's Net contains the following sub-modules:

    *   agent_pop.py: code dealing with managing and tracking the population of agents.
    *   data_methods.py: getting data about a run out to other programs.
    *   display_methods.py: code dealing with graphs.
    *   entity.py: a mode that describes and environment, agent or object.
    *   entry_point.py: common code for entry points to modules.
    *   env.py: the base environment all others inherit from.
    *   grid_agent.py: the base class for all agents located on a grid.
    *   grid_env.py: the base env for all grid envs.
    *   main_menu.py: the default main menu for all models.
    *   markov.py: basic code for Markov chains.
    *   markov_agent.py: the base class for all Markov agents.
    *   markov_env.py: the base class for Markov environments.
    *   menu.py: contains the basic menuing capabilities.
    *   node.py: the absolute base of everything in Indra's Net.
    *   prop_args.py: the class managing parameters to a model.
    *   spatial_agent.py: the base class for all agents located in space.
    *   spatial_env.py: the base env for spatial models (including grid).
    *   two_pop_model.py: two population models (wolf-sheep etc.) subclass from here.
    *   user.py: manages interacting with the user.
    *   utils.py: miscellaneous useful functions.
    *   vector_space.py: code dealing with vector spaces.
    *   vs_agent.py: base class for agents inhabiting a vector space.

"""
