"""
Various models

This module contains miscellaneous example models built using Indra.

The module contains:
    * __main__.py: an entry point into the module.
    * basic_model.py: just exercises the most basic agent capabilities.
    * basic_run.py: runs the above model.
    * coop_model.py: Krugman's babysitting co-op model.
    * coop_run.py: runs the above model.
    * fashion_model.py: Adam Smith's fashion model.
    * fashion_run.py: runs the above model.
    * fmarket_model.py: financial market oscillations.
    * fmarket_run.py: runs the above model.
    * forestfire_model.py: the classic forest fire model.
    * forestfire_run.py: runs the above model.
    * grid_model.py: just exercises the most basic grid capabilites.
    * grid_run.py: runs the above model.
    * menu_model.py: for testing menu capabilities.
    * menu_run.py: runs the above model.
    * obst_model.py: detecting obstacles in the env.
    * obst_run.py: runs the above model.
    * sand_model.py: an Abelian sandpile model.
    * sand_run.py: runs the above model.
    * spatial_model.py: the most basic spatial model.
    * spatial_run.py: runs the above model.
    * timer_model.py: for timing models.
    * timer_run.py: runs the above model.
    * two_pop_model.py: a base for other two population models, like wolfsheep.
    * two_pop_run.py: runs the above model.
    * wolfsheep_model.py: an elementary predator-prey model.
    * wolfsheep_run.py: runs the above model.
"""
