"""
Big Box

The bigbox module contains code relating to studying the relationship between
big box retailers and mom-and-pop shops.

The module contains:
    * big_box_model.py: the basic model.
    * big_box_run.py: collects parameters for the big box model and runs it.
"""
