"""
Schelling models

This module contains models based on the work of Thomas Schelling.

The module contains:
    * auditorium_model.py: Schelling's auditorium filling puzzle.
    * auditorium_run.py: runs the above model.
    * height_model.py: Schelling's genetically engineered height model.
    * height_run.py: runs the above model.
    * segregation_model.py: Schelling's model of neighborhood segregation.
    * segregation_run.py: runs the above model.
"""
